import serpent
import hashlib
from binascii import a2b_hex, b2a_hex
from sys import argv

class PublicRSAKey:
	def __init__(self, e: int, n:int):
		self.e = e
		self.n = n

def md5(s: bytes) -> bytes:
	return hashlib.md5(s).digest()

def RSAPublicDecrypt(m: int, e: int, n: int) -> int:
	# Calculate (m^e)%n
	return pow(m,e,n)

def unsign_data(data: bytes, key: PublicRSAKey) -> tuple:
	# return (True, data) if verify successfully
	# else return (False, )
	if len(data) <= 128:
		return (False, )
	ds = data[-128:]
	ds = int.from_bytes(ds, 'big')
	p = RSAPublicDecrypt(ds, key.e, key.n)
	p = p.to_bytes(128, 'big')
	if not p.startswith(b'\x00\x01'):
		return (False, )
	i = 2
	while i < len(p):
		if p[i] == 0:
			break
		i = i + 1
	if i == len(p):
		return (False, )
	p = p[i + 1:]
	signed_md5_hash = p[0:16]
	serpent_key = p[16:32]
	decrypted_size = int.from_bytes(p[32:36], 'little')
	decrypted_data = serpent.serpent_cbc_decrypt(serpent_key, data[:-128], b'\x00'*16)[:decrypted_size]
	md5_hash = md5(decrypted_data)
	if signed_md5_hash == md5_hash:
		return (True, decrypted_data)
	return (False, )

#rsa_key = PublicRSAKey(0x10001, )

# 31326432`38393039
if __name__ == '__main__':
	if len(argv) != 2:
		print ('[+] python dec.py <filename>')
		exit(1)
	n = \
	"c3da263df172293373b0431e"+\
	"e00bac4c3db723bee2d9ccc0a7ef8d03"+\
	"68c33c577df7e64f09503437e9178533"+\
	"c9f3b4d4eebd7fe1075e2e553939d43c"+\
	"25eb8a89a5fd7ad5f8a52c20713ae878"+\
	"cf2b1f322acfe8b7c55dad60b3520614"+\
	"19fa713c903d9efc36baf95185880d03"+\
	"ec165a51186cf1c323bc58c40b85fcbc"+\
	"7fa162ad"
	n = int.from_bytes(a2b_hex(n), 'big')
	e = 0x10001
	rsa_key = PublicRSAKey(e, n)
	with open(argv[1], 'rb') as f:
		data = f.read()
	res = unsign_data(data, rsa_key)
	if res[0] == True:
		with open(argv[1] + '.decrypt', 'wb') as f:
			f.write(res[1])
		print ('[+] Unsigned %s (%d bytes -> %d bytes) !' % (argv[1], len(data), len(res[1])))
	else:
		print ('[+] Wrong signature !')
